'''
@since: 5 Feb 2016
@author: Prashant Bhosale <pbhosale@lenovo.com>, Girish Kumar <gkumar1@lenovo.com>
@license: Lenovo License
@copyright: Copyright 2016, Lenovo
@organization: Lenovo
@summary: This module provides scriptable interfaces and scriptable python shell.
'''

import code
import logging
import sys

from pylxca import __version__
from pylxca.pylxca_cmd import lxca_ishell
from pylxca.pylxca_cmd.lxca_cmd import fanmuxes

# shell is a global variable
SHELL_OBJ = None
LOGGER = logging.getLogger(__name__)


def pyshell(shell=lxca_ishell.InteractiveShell()):
    '''
    @summary: this method provides scriptable python shell
    '''
    global SHELL_OBJ
    SHELL_OBJ = shell
    SHELL_OBJ.set_ostream_to_null()


def set_interactive():
    '''
    @summary: This method set the shell in interactive mode
    '''
    ns = {"connect": connect,
          "disconnect": disconnect,
          "chassis": chassis,
          "cmms": cmms,
          "fans": fans,
          "fanmuxes": fanmuxes,
          "switches": switches,
          "powersupplies": powersupplies,
          "nodes": nodes,
          "scalablesystem": scalablesystem,
          "discover": discover,
          "manage": manage,
          "unmanage": unmanage,
          #"jobs": jobs,
          "users": users,
          "lxcalog": lxcalog,
          "ffdc": ffdc,
          "updatecomp": updatecomp,
          "updatepolicy": updatepolicy,
          "updaterepo": updaterepo,
          "configpatterns": configpatterns,
          "configprofiles": configprofiles,
          "configtargets": configtargets,
          "tasks": tasks,
          "manifests": manifests,
          "osimages": osimages,
          "resourcegroups": resourcegroups,
          "storedcredentials": storedcredentials,
          "managementserver": managementserver,
          "license": license,
          "help": help}
    ns.update()
    global __version__
    code.interact('Interactive Python Shell for Lenovo XClarity Administrator v' + __version__ +
                  '\nType "dir()" or "help(lxca command object)" for more information.', local=ns)


def connect(*args, **kwargs):
    '''

@summary:
    Use this function to connect to Lenovo XClarity Administrator
    run this function as
    con_variable = connect( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['url', 'user', 'pw', 'noverify']

@param
    The parameters for this command are as follows
        con          Connection Object to Lenovo XClarity Administrator
        url          url to Lenovo XClarity Administrator Example. https://a.b.c.d
        user         User Id to Authenticate Lenovo XClarity Administrator
        pw           Password to Authenticate Lenovo XClarity Administrator
        noverify     flag to indicate to not verify server certificate

@example 
    con1 = connect( url = "https://10.243.12.142",user = "USERID", pw = "Password", noverify = "True")
    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name
    keylist = ['url', 'user', 'pw', 'noverify']
    if len(args) == 0 and len(kwargs) == 0:
        return

    for i in range(len(args)):
        kwargs[keylist[i]] = args[i]

    con = SHELL_OBJ.handle_input_args(command_name, args=args, kwargs=kwargs)

    return con


def disconnect(*args, **kwargs):
    '''

@summary:
    Use this function to disconnect from Lenovo XClarity Administrator
    run this function as
    disconnect()


     it also reset internal current connection to original connection this is used in api version
     to retain origianal connection if we are disconnecting other than current connection

     i.e
     con1 = connect(...)
     con2 = connect(...)
     con3 = connect(...)
     con4 = connect(...)  current internal connection is con4

     disconnect(con2)  will keep current connection to con4
     disconnect(con4) or disconnect() will set current connection to None


@param
    The parameters for this command are as follows

        con      Connection Object to Lenovo XClarity Administrator

@example 
    disconnect()
    '''
    global SHELL_OBJ

    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None

    long_short_key_map = {}
    keylist = ['con']
    optional_keylist = ['con']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict, False)
    return out_obj


def cmms(*args, **kwargs):
    '''

@summary:
    Use this function to get CMMs information
    run this function as  

    data_dictionary = cmms( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con', 'uuid', 'chassis']

@param
    The parameters for this command are as follows 

    con       Connection Object to Lenovo XClarity Administrator
    uuid      cmm uuid
    chassis   chassis uuid  

@example 
    cmm_list = cmms( con = con1, uuid = 'fc3058cadf8b11d48c9b9b1b1b1b1b57')
    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None

    long_short_key_map = {'uuid': 'u', 'chassis': 'c'}
    keylist = ['con', 'uuid', 'chassis']
    optional_keylist = ['con', 'uuid', 'chassis']
    mutually_exclusive_keys = ['uuid', 'chassis']
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def chassis(*args, **kwargs):
    '''

@summary:
    Use this function to get Chassis information
    run this function as  

    data_dictionary = chassis( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con', 'uuid', 'status']

@param
    The parameters for this command are as follows 

    con        Connection Object to Lenovo XClarity Administrator
    uuid       chassis uuid
    status     chassis manage status (managed/unmanaged)


@example 
    chassis_list = chassis( con = con1, uuid = 'fc3058cadf8b11d48c9b9b1b1b1b1b57')
    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None

    long_short_key_map = {'uuid': 'u', 'status': 's'}
    keylist = ['con', 'uuid', 'status']
    optional_keylist = ['con', 'uuid', 'status']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def fans(*args, **kwargs):
    '''

@summary:
    Use this function to get fans information
    run this function as  

    data_dictionary = fans( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con', 'uuid', 'chassis']

@param
    The parameters for this command are as follows 

    con           Connection Object to Lenovo XClarity Administrator
    uuid          uuid of fan
    chassis       chassis uuid

@example 
    fans_list = fans( con = con1, uuid = 'fc3058cadf8b11d48c9b9b1b1b1b1b57')
    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None

    long_short_key_map = {'uuid': 'u', 'chassis': 'c'}
    keylist = ['con', 'uuid', 'chassis']
    optional_keylist = ['con', 'uuid', 'chassis']
    mutually_exclusive_keys = ['uuid', 'chassis']
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def fanmuxes(*args, **kwargs):
    '''

@summary:
    Use this function to get fanmuxes information
    run this function as  

    data_dictionary = fanmuxes( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','uuid','chassis']

@param
    The parameters for this command are as follows 

    con           Connection Object to Lenovo XClarity Administrator
    uuid          uuid of fanmux
    chassis       chassis uuid

@example 

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None

    long_short_key_map = {'uuid': 'u', 'chassis': 'c'}
    keylist = ['con', 'uuid', 'chassis']
    optional_keylist = ['con', 'uuid', 'chassis']
    mutually_exclusive_keys = ['uuid', 'chassis']
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def nodes(*args, **kwargs):
    '''

@summary:
    Use this function to get nodes information
    run this function as  

    data_dictionary = nodes( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','uuid','chassis','status', 'modify', 'metrics']

@param
    The parameters for this command are as follows 

    con           Connection Object to Lenovo XClarity Administrator
    uuid          uuid of node
    chassis       chassis uuid
    status        nodes manage status (managed/unmanaged)
    modify        JSON object of modifyable node properties
    metrics       flag to fetch metrics of all nodes or metrics of a node belonging to the provided uuid

@example 

    nodes(uuid="FAA6E3D494E511E6A0739B91ED670CE8",modify='{"location":{"location": "new location", "rack": "rack 5","lowestRackUnit": 3}}')
    
    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None

    long_short_key_map = {'uuid': 'u', 'chassis': 'c', 'status': 's', 'modify': 'm', 'metrics': 'x'}
    keylist = ['con', 'uuid', 'chassis', 'status', 'modify', 'metrics']
    optional_keylist = ['con', 'uuid', 'chassis', 'status', 'modify', 'metrics']
    mutually_exclusive_keys = ['uuid', 'chassis']
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def switches(*args, **kwargs):
    '''

@summary:
    Use this function to get switches information
    run this function as  

    data_dictionary = switches( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','uuid','chassis','ports','action']

@param
    The parameters for this command are as follows 

    con      Connection Object to Lenovo XClarity Administrator
    uuid          uuid of switch
    chassis       chassis uuid
    ports         empty ports string list all ports for uuid, comma separated ports
    action        enable/disable ports

@example 

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None

    # other parameter don't have short option
    long_short_key_map = {'uuid': 'u', 'chassis': 'c'}
    keylist = ['con', 'uuid', 'chassis', 'ports', 'action']
    optional_keylist = ['con', 'uuid', 'chassis', 'ports', 'action']
    mutually_exclusive_keys = ['uuid', 'chassis']
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def powersupplies(*args, **kwargs):
    '''

@summary:
    Use this function to get powersupplies information
    run this function as  

    data_dictionary = powersupplies( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','uuid','chassis']

@param
    The parameters for this command are as follows 

    con      Connection Object to Lenovo XClarity Administrator
    uuid          uuid of power supply
    chassis       chassis uuid

@example 

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None
    long_short_key_map = {'uuid': 'u', 'chassis': 'c'}
    keylist = ['con', 'uuid', 'chassis']
    optional_keylist = ['con', 'uuid', 'chassis']
    mutually_exclusive_keys = ['uuid', 'chassis']
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def scalablesystem(*args, **kwargs):
    '''

@summary:
    Use this function to get scalablesystem information
    run this function as  

    data_dictionary = scalablesystem( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','id','type']

@param
    The parameters for this command are as follows 

    con      Connection Object to Lenovo XClarity Administrator
    id        scalable complex id
    type      type (flex/rackserver)

@example 

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None

    long_short_key_map = {'id': 'i', 'type': 't'}
    keylist = ['con', 'id', 'type']
    optional_keylist = ['con', 'id', 'type']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def discover(*args, **kwargs):
    '''

@summary:
    Use this function to discover endpoint from Lenovo XClarity Administrator
    run this function as  

    data_dictionary = discover( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','ip','job']

@param
    The parameters for this command are as follows 

    con    Connection Object to Lenovo XClarity Administrator
    ip     One or more IP addresses for each endpoint to be discovered.
    job    Job ID of discover request


@example

    For Getting Maangement job status

        job_data = discover(con=con1,job=jobid)

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None

    long_short_key_map = {'ip': 'i', 'job': 'j'}
    keylist = ['con', 'ip', 'job']
    optional_keylist = ['con', 'ip', 'job']
    mutually_exclusive_keys = ['ip', 'job']
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)
    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def manage(*args, **kwargs):
    '''

@summary:
    Use this function to manage endpoint from Lenovo XClarity Administrator
    run this function as  

    data_dictionary = manage( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','subcmd','ip','user','pw','rpw','job','force', 'storedcredential_id']

@param
    The parameters for this command are as follows 

        con      Connection Object to Lenovo XClarity Administrator
        subcmd
        ip       One or more IP addresses for each endpoint to be managed.
        user     user ID to access the endpoint
        pw       The current password to access the endpoint.
        rpw      The recovery password to be used for the endpoint.
        force     force manage
        job       Job ID of existing manage request
        storedcredential_id    Store Crendential Id to be used for device manage operation
                        if this is provided user is not required
        Note : mp, type and epuuid parameters are dedpriciated and only kept for backword compatibility. 

@example 

        jobid = manage(con=con1, subcmd='device', ip="10.243.6.68",user="USERID",pw="PASSW0RD",rpw="PASSW0RD")
        jobid = manage(con=con1, subcmd='device', ip="10.243.6.68",storedcredintail_id="12")

    or with named variable it can be represented as

        jobid = manage(con= con1, subcmd='device', ip="10.243.6.68",user="USERID","PASSW0RD","PASSW0RD",True)
        jobid = manage(con1, subcmd='device', i="10.243.4.16", u='USERID', p='Purley44LEN', r='Purley55LEN', f='True')

        Using storedcredential id for Rackswitch
        jobid = manage(con=con1, subcmd='device', i='10.240.157.111', s='402', f='True')

        Using storedcredential id for Rackswitch Server
        jobid = manage(con=con1, subcmd='device',i="10.243.4.16", r='Purley55LEN', s='404', f='True')


    For Getting Maangement job status

        jobid = manage(con=con1, subcmd='job_status', job="12")
    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None

    long_short_key_map = {'ip': 'i', 'user': 'u', 'pw': 'p',
                          'rpw': 'r', 'job': 'j', 'force': 'f', 'storedcredential_id': 's'}
    keylist = ['con', 'subcmd', 'ip', 'user', 'pw',
               'rpw', 'job', 'force', 'storedcredential_id']
    optional_keylist = ['con', 'ip', 'user', 'pw',
                        'rpw', 'job', 'force', 'storedcredential_id']
    mutually_exclusive_keys = ['ip', 'job']
    mandatory_options_list = {'job': []}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def unmanage(*args, **kwargs):
    '''

@summary:
    Use this function to unmanage endpoint from Lenovo XClarity Administrator
    run this function as  

    data_dictionary = unmanage( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','subcmd','ep','force','job']

@param
    The parameters for this command are as follows 
        subcmd      device \ job_status
        ep          one or more endpoints to be unmanaged.
                    This is comma separated list of multiple endpoints, each endpoint should
                    contain endpoint information separated by semicolon.
                    endpoint's IP Address(multiple addresses should be separated by #), UUID of the endpoint and
                    Type of endpoint to be unmanaged ,This can be one of the following values:
                          Chassis
                          ThinkServer
                          Storage
                          Rackswitch
                          Rack-Tower
                          Edge
        force       Indicates whether to force the unmanagement of an endpoint (True/False)
        job         Job ID of unmanage request

@example 
    endpoint = '10.240.195.39;D31C76F0302503B50010D21DE03A0523;Rack-Tower'
    unmanage(con_lxca, subcmd='device', ep=endpoint)
    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None

    long_short_key_map = {'ep': 'e', 'job': 'j', 'force': 'f'}
    keylist = ['con', 'subcmd', 'ep', 'force', 'job']
    optional_keylist = ['con', 'ep', 'force', 'job']
    mutually_exclusive_keys = ['ep', 'job']
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def configpatterns(*args, **kwargs):
    '''

@summary:
    Use this function to Retrieve information and deploy all server and category patterns
            that have been defined in the Lenovo XClarity Administrator

    run this function as  

    data_dictionary = configpatterns( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','subcmd','id', 'includeSettings', 'endpoint','restart','type', pattern_update_dict, name, status]

@param
    The parameters for this command are as follows 
        subcmd     list, apply, import , status
        id          The unique ID that was assigned when the server pattern was created
        name        name of pattern , this is used for apply
        endpoint    List of one or more UUIDs for the target servers,If a target is an empty bay,
                      specify the location ID; otherwise, specify the server UUID

        restart     When to activate the configurations. This can be one of the following values:
                      defer - Activate IMM settings but do not restart the server.
                      immediate - Activate all settings and restart the server immediately.
                      pending - Manually activate the server profile and restart the server.

        type        Type of the server, It can be one of the following
                      flex -  Flex System Placeholder chassis empty bays
                      node
                      rack
                      tower

        pattern_update_dict  dictionary of category_pattern to import.

        status      check config status for given uuid in endpoint
                    True

@example 

    list  all configpatterns
        rep = configpatterns(con, subcmd = 'list')

    list particular configpattern
        rep = configpatterns(con, subcmd = 'list', id="53")

    list particular configpattern with includeSettings
        rep = configpatterns(con, subcmd = 'list', id="53", includeSettings="True")

    import SystemInfo category pattern

        py_dict = {
            "template_type": "SystemInfo",
            "template": {
                "contact": "contact",
                "description": "Pattern created by testAPI",
                "location": "location",
                "name": "Learned-System_Info-19",
                "systemName": {
                    "autogen": "Disable",
                    "hyphenChecked": False
                },
                "type": "SystemInfo",
                "uri": "/config/template/61",
                "userDefined": True
            }
        }
        import json
        my_json = json.dumps(py_dict)
        rep = configpatterns(con_lxca, 'import', pattern_update_dict = my_json )
    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None

    # some of them don't have short options
    long_short_key_map = {'id': 'i', 'endpoint': 'e', 'restart': 'r',
                          'type': 't', 'name': 'n', 'pattern_update_dict': 'p'}
    keylist = ['con', 'subcmd', 'id', 'includeSettings', 'endpoint',
               'restart', 'type', 'pattern_update_dict', 'name']
    optional_keylist = ['con', 'id', 'includeSettings', 'endpoint',
                        'restart', 'type', 'pattern_update_dict', 'name']
    mutually_exclusive_keys = ['id', 'pattern_update_dict']
    mandatory_options_list = {'id': [], 'pattern_update_dict': []}
    #                          'includeSettings': ['id']}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def configprofiles(*args, **kwargs):
    '''

@summary:
    Use this function to Retrieve information server configuration profiles
            that have been defined in the Lenovo XClarity Administrator

    run this function as  

    data_dictionary = configprofiles( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con', 'subcmd', 'id', 'name', 'endpoint', 'restart', 'powerdown', 'resetimm', 'resetswitch', 'force']

@param
    The parameters for this command are as follows 
        subcmd      list, rename, activate, unassign, delete
        id          The unique ID that was assigned when the server profile was created
        name        profile name
        endpoint    endpoint  UUID of the server or location id for flex system
        restart     restart server to activate profile ( immediate / defer )

                    options for unassign
        powerdown   powerdown server
        resetimm    reset IMM
        resetswitch  Identifies whether to reset the switch internal port settings to default values
        force       force unassign operation

@example
    rep = configprofiles(con_lxca, 'list')
    rep = configprofiles(con_lxca, 'list','3')
    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None
    # some of keys don't have short option
    long_short_key_map = {'id': 'i', 'name': 'n', 'endpoint': 'e', 'restart': 'r',
                          'powerdown': 'p', 'force': 'f'}
    keylist = ['con', 'subcmd', 'id', 'name', 'endpoint',
               'restart', 'powerdown', 'resetimm', 'resetswitch', 'force']
    optional_keylist = ['con', 'id', 'name', 'endpoint',
                        'restart', 'powerdown', 'resetimm', 'resetswitch', 'force']
    mutually_exclusive_keys = []
    mandatory_options_list = {'id': [], 'endpoint': ['restart']}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)

    #remove dummy field added for view
    if 'dummy' in out_obj:
        out_obj.pop('dummy')
    return out_obj


    return out_obj


def configtargets(*args, **kwargs):
    '''

@summary:
    Use this function to get config pattern targets from Lenovo XClarity Administrator
    run this function as  

    data_dictionary = configtargets( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','id']

@param
    The parameters for this command are as follows 
        id    config target id

@example 

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None
    long_short_key_map = {'id': 'i'}
    keylist = ['con', 'id']
    optional_keylist = ['con']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def updatepolicy(*args, **kwargs):
    '''

@summary:
    Use this function to read Firmwar update Policy from Lenovo XClarity Administrator
    run this function as  

    data_dictionary = updatepolicy( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con', 'subcmd', 'info','job','uuid',policy','Type']

@param
    The parameters for this command are as follows 
    subcmd  list,query, assign , status
    info    Specifies the type of information to return. This can be one of the following values:
                FIRMWARE- Returns information about firmware that is applicable to each managed endpoint
                RESULTS- Returns persisted compare result for servers to which a compliance policy is assigned
                NAMELIST -  Returns the available compliance policies
    jobid    Job ID of assign compliance policy operation

    uuid     UUID of the device to which you want to assign the compliance policy

    policy   Policyname, Name of the compliance-policy to be assigned to device

    Type     Device type. This can be one of the following values.
                    CMM - Chassis Management Module
                    IOSwitch - Flex switch
                    RACKSWITCH - RackSwitch switch
                    STORAGE - Lenovo Storage system
                    SERVER - Compute node or rack server

@example

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None
    long_short_key_map = {'info': 'i', 'job': 'j',
                          'uuid': 'u', 'policy': 'p', 'type': 't'}
    keylist = ['con', 'subcmd', 'info', 'job', 'uuid', 'policy', 'type']
    optional_keylist = ['con', 'info', 'job', 'uuid', 'policy', 'type']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def updaterepo(*args, **kwargs):
    '''

@summary:
    Use this function to get repository info from Lenovo XClarity Administrator
    run this function as  

    data_dictionary = updaterepo( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con', 'subcmd', 'key', 'mt', 'scope', 'fixids', 'type']

@param
    The parameters for this command are as follows 


    subcmd    The action to take. This can be one of the following values.
                query - Get  info using key parameter
                read - Reloads the repository files. The clears the update information in cache and reads the update file again from the repository.
                refresh - Retrieves information about the latest available firmware updates from the Lenovo Support website,
                         and stores the information to the firmware-updates repository.
                acquire - Downloads the specified firmware updates from Lenovo Support website, and stores the updates to the firmware-updates repository.
                delete - Deletes the specified firmware updates from the firmware-updates repository.
                export.not supported

    key    Returns the specified type of update. This can be one of the following values.
                supportedMts - Returns a list of supported machine types
                size - Returns the repository size
                lastRefreshed - Returns the timestamp of the last repository refresh
                importDir - Returns the import directory for the repository.
                publicKeys - Returns the supported signed keys
                updates - Returns information about all firmware updates
                updatesByMt - Returns information about firmware updates for the specified machine type
                updatesByMtByComp - Returns the update component names for the specified machine type


     mt        comma separated machine types
     scope     scope of operation
     fixids    comma separated fixids
     type      filetype for PUT opertaion
@example 
     rep = updaterepo(con, "query", k="size")
     rep = updaterepo(con, subcmd = "read")
     rep = updaterepo(con_lxca, subcmd = "read")
     rep = updaterepo(con_lxca, subcmd = "acquire", mt="7903")
    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None

    long_short_key_map = {'key': 'k', 'mt': 'm',
                          'scope': 's', 'fixids': 'f', 'type': 't'}
    keylist = ['con', 'subcmd', 'key', 'mt', 'scope', 'fixids', 'type']
    optional_keylist = ['con', 'key', 'mt', 'scope', 'fixids', 'type']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def updatecomp(*args, **kwargs):
    '''

@summary:
    Use this function to update firmware of endpoint from Lenovo XClarity Administrator
    run this function as

    data_dictionary = updatecomp( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

    USAGE:

        keylist = ['con','subcmd', 'query','mode','action','cmm','switch','server','storage','dev_list']

@param
    The parameters for this command are as follows
    subcmd  info|apply
    query   The data to return. This can be one of the following values.
                components - Returns a list of devices and components that can be updated.
                status - Returns the status and progress of firmware updates. This is the default value

    mode    Indicates when to activate the update. This can be one of the following values.
                immediate - Uses Immediate Activaton mode when applying firmware updates to the selected endpoints.
                delayed - Uses Delayed Activaton mode when applying firmware updates to the selected endpoints.
                prioritized. Firmware updates on the baseboard management controller are activated immediately

    action  The action to take. This can be one of the following values.
                apply - Applies the associated firmware to the submitted components.
                power - Perform power action on selected endpoint.
                cancelApply - Cancels the firmware update request to the selected components.

    cmm     cmms information
    switch  switch information
    server  servers information
    storage storages information
    dev_list  - update all updateable components
            For action = apply / applyBundle / cancelApply, Device information should contain following data separated by comma
                UUID - UUID of the device
                Fixid - Firmware-update ID of the target package to be applied to the component.
                         If not provided assigned policy would be used.
                Component - Component name

            For action = power, Device information should contain following data separated by comma
                UUID - UUID of the device
                powerState - One of the power state values. Possible values per device type are
                    Server: powerOn, powerOff, powerCycleSoft, powerCycleSoftGraceful, powerOffHardGraceful
                    Switch: powerOn, powerOff, powerCycleSoft
                    CMM: reset
                    Storage:powerOff,powerCycleSoft

@example
Applying firmware update to server
endpoint = "38B1DC62084411E88C7A0A94EF4EC2EF,lnvgy_fw_lxpm_pdl116o-1.40_anyos_noarch,LXPM Diagnostic Software"
rep = updatecomp(con, 'apply', action='apply', mode='immediate', server=endpoint)

Applying firmware update to ThinkSystem SR635 and SR655
endpoint = "53619A91874311E9A3E3E086720E6C33"
rep = updatecomp(con, 'apply', action='applyBudle', server=endpoint)

Applying firmware update using dev_list json format

dev_list = {
   "DeviceList": [{
      "ServerList": [{
         "UUID": "38B1DC62084411E88C7A0A94EF4EC2EF",
         "Components": [{
            "Fixid": "lnvgy_fw_lxpm_pdl116o-1.40_anyos_noarch",
            "Component": "LXPM Diagnostic Software"
         }]
      }]
   },
   {
      "CMMList": []
   },
   {
      "SwitchList": []
   },
   {
      "StorageList": []
   }]
}

dev_json = json.dumps(dev_list)
rep = updatecomp(con, 'apply', action='apply', mode='immediate', dev_list=dev_json)

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None
    long_short_key_map = {'query': 'q', 'mode': 'm', 'action': 'a', 'cmm': 'c', 'switch': 'w', 'server': 's',
                          'storage': 't', 'dev_list': 'l'}
    keylist = ['con', 'subcmd', 'query', 'mode', 'action', 'cmm',
               'switch', 'server', 'storage', 'dev_list']
    optional_keylist = ['con', 'query', 'mode', 'action',
                        'cmm', 'switch', 'server', 'storage', 'dev_list']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def users(*args, **kwargs):
    '''

@summary:
    Use this function to get users data from Lenovo XClarity Administrator
    run this function as  

    data_dictionary = users( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','id']

@param
    The parameters for this command are as follows 

        id    unique ID of the user to be retrieved

@example 

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None
    long_short_key_map = {'id': 'i'}
    keylist = ['con', 'id']
    optional_keylist = ['con', 'id']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def ffdc(*args, **kwargs):
    '''

@summary:
    Use this function to Collect and export specific endpoint data 
        from Lenovo XClarity Administrator

    run this function as  

    data_dictionary = ffdc( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','uuid']

@param
    The parameters for this command are as follows 

        uuid    UUID of the target endpoint this is manadatory parameter

@example
    ffdc(con = lxca_con, uuid='UUID of endpoint")

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None
    long_short_key_map = {'uuid': 'u'}
    keylist = ['con', 'uuid']
    optional_keylist = ['con', 'uuid']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def log(*args, **kwargs):
    '''

@summary:
    Use this function to get Lenovo XClarity Administrator LOG information
    run this function as

    data_dictionary = log( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','lvl']

@param
    The parameters for this command are as follows

        lvl  log level to be set
        Possible Log Levels, Please use following values to set desired log level.

        DEBUG:        Detailed information, typically of interest only when diagnosing problems.
        INFO:        Confirmation that things are working as expected.
        WARNING:    An indication that something unexpected happened, or indicative of some problem in the near future.
        ERROR:        Due to a more serious problem, the software has not been able to perform some function.
        CRITICAL:    A serious error, indicating that the program itself may be unable to continue running.

@example

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None

    long_short_key_map = {'lvl': 'l'}
    keylist = ['con', 'lvl']
    optional_keylist = ['con', 'lvl']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def lxcalog(*args, **kwargs):
    '''

@summary:
    Use this function to get Lenovo XClarity Administrator LOG information
    run this function as  

    data_dictionary = lxcalog( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','filter']

@param
    The parameters for this command are as follows 

        filter  filter for the event

@example 

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None

    long_short_key_map = {'filter': 'f'}
    keylist = ['con', 'filter']
    optional_keylist = ['con', 'filter']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def jobs(*args, **kwargs):
    '''

@summary:
    Use this function to get jobs information from Lenovo XClarity Administrator
    run this function as  

    data_dictionary = jobs( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con','id','uuid','state','cancel','delete']

@param
    The parameters for this command are as follows 

        id=         job id
        uuid=       uuid of endpoint for which jobs should be retrieved
        state=      job state to retrieve jobs in specified state.
                      The state can be one of the following
                      Pending
                      Running
                      Complete
                      Cancelled
                      Running_With_Errors
                      Cancelled_With_Errors
                      Stopped_With_Error
                      Interrupted
        cancel=     cancel job of specified id
        delete=     delete job of specified id

@example 

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None

    long_short_key_map = {'id': 'i', 'uuid': 'u',
                          'state': 's', 'cancel': 'c', 'delete': 'd'}
    keylist = ['con', 'id', 'uuid', 'state', 'cancel', 'delete']
    optional_keylist = ['con', 'id', 'uuid', 'state', 'cancel', 'delete']
    mutually_exclusive_keys = ['id', 'cancel', 'delete']
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def manifests(*args, **kwargs):
    '''

@summary:
    Use this function to send solution manifest to and retreive manifests from Lenovo XClarity Administrator
    run this function as  

    data_dictionary = manifests( conn_handle, input_args_dictionary{key,value} )

    Where KeyList is as follows

        keylist = [id','file']

@param
    The parameters for this command are as follows 

        id=         solution id
        file=       path to manifest file

@example 

    '''
    global SHELL_OBJ

    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None

    long_short_key_map = {'id': 'i', 'file': 'f'}
    keylist = ['con', 'id', 'file']
    optional_keylist = ['con', 'file']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    # return out_obj
    return True


def tasks(*args, **kwargs):
    '''

@summary:
    Use this function to get tasks information
    run this function as

     = tasks( con, data_dictionary)

    Where data_dictionary contain input arguments as follows

        keylist = ['jobUID','children','action', 'updateList']

@param
    The parameters for this command are as follows

    con      Connection Object to Lenovo XClarity Administrator
    jobUID          uuid of job
    children        result will include children if True
    action          cancel/update/create
    updateList      required for update action , string containing list of update

@example
    update_list = [{"jobUID":"9","percentage":50}]
    str_u = str(update_list)
    rep = tasks(con_lxca, a = 'update', u = str_u)
    '''
    global SHELL_OBJ
    con = None
    param_dict = {}

    command_name = sys._getframe().f_code.co_name

    param_dict = {}
    con = None

    long_short_key_map = {'jobUID': 'j', 'children': 'c',
                          'action': 'a', 'updateList': 'u', 'template': 't'}
    keylist = ['con', 'jobUID', 'children', 'action', 'updateList','template']
    optional_keylist = ['con', 'jobUID', 'children', 'action', 'updateList','template']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def resourcegroups(*args, **kwargs):
    '''

@summary:
    Use this function to Create, modify, delete or read resource group from Lenovo XClarity Administrator
    run this function as

    data_dictionary = resourcegroups( con_handle,uuid, name, desc, type, solutionVPD, members, criteria )

    Where KeyList is as follows

        keylist = ['uuid', 'name','description','type','solutionVPD','members','criteria']

@param
    The parameters for this command are as follows

        uuid=         UUID of already created Group 
        name=         Name of Resource Group
        desc=         Description of Resource Group
        type=         Type of Resource Group. <{"static", "dynamic", "solution"}>,
        solutionVPD": { "id": <UUID string>,
                        "machineType": <machine type string>,
                        "model": <model string>,
                        "serialNumber": <serial number string>,
                        "manufacturer": <string>
                      },

        "members": [ "uri","uri",....],
        "criteria":[]

@example
    list all resourcegroups
    rep = resourcegroups(con_lxca, 'list')

    list criteria properties for dynamic groups
    rep = resourcegroups(con_lxca, 'criteriaproperties')

    For creating dynamic group
    criteria = {
				u'parent': u'root',
				u'value': None,
				u'criteria': [{
						u'operator': u'contains',
						u'value': u'test',
						u'property': u'hostname',
						u'id': u'1001',
						u'parent': u'lxca_customUI_resourceViews_allGroupsPage_editGroupDynamicPage_2'
					}
				],
				u'operator': u'AND',
				u'property': None,
				u'id': u'root'
			}

    criteria_json = json.dumps(criteria)
    print criteria_json
    rep = resourcegroups(con_lxca, 'create', n="TEST_DYNAMIC", d="TRIAL_GROUP", t='dynamic', c=criteria_json)

    Updating dynamic group
    rep = resourcegroups(con_lxca, 'update', u="5C5AB42D94C6A719BEF2A375", n="R1_GROUP", d="TRIAL_GROUP modified", t='dynamic', c=criteria_json)

    Delete resouregroup
    rep = resourcegroups(con_lxca, 'delete', u="5C5BC6EA90F54D074FC7BC0D")

    Create solution group supported for api only for uhm
    solutionVPD = {
               'id': '59A54997C18DCF0594A8CCD1',
               'machineType': 'TESTMTM',
               'model': 'TESTMODEL',
               'serialNumber': 'TESTSERIAL',
               'manufacturer': 'LENOVO'}
    members = []
    criteria = []

    rep = resourcegroups(con_lxca, 'create', n="TEST_solution", d="Test_GROUP", t='solution', s=solutionVPD, m=members, c=criteria)


    '''
    global SHELL_OBJ
    con = None
    param_dict = {}

    command_name = sys._getframe().f_code.co_name

    long_short_key_map = {'uuid': 'u', 'name': 'n', 'description': 'd', 'type': 't', 'solutionVPD': 's',
                          'members': 'm', 'criteria': 'c'}

    keylist = ['con', 'subcmd', 'uuid', 'name', 'description',
               'type', 'solutionVPD', 'members', 'criteria']
    optional_keylist = ['con', 'uuid', 'name', 'description',
                        'type', 'solutionVPD', 'members', 'criteria']
    mutually_exclusive_keys = []
    mandatory_options_list = {'uuid': [], 'name': ['type']}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist, mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    LOGGER.debug("resourcegroups %s" %str(param_dict))
    if 'type' in param_dict:
        if 'solution' in param_dict['type']:
            out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict, False)
        else:
            out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    else:
        out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                    mutually_exclusive_keys, param_dict, *args, **kwargs):
    '''
     this function will create param_dict and con from args and kwargs, param_dict will have only long options for keys,
     it will convert short option to long option key and finally validate parameters
    :param arglist: list of arguments derived from args

    :param keylist: keylist of name of fields
    :param mandatory_options_list:
    :param optional_keylist:
    :param mutually_exclusive_keys:
    :param param_dict: append to param_dict
    :return: connection object
    '''
    arglist = list(args)
    arglist = arglist[::-1]
    con = None
    for key in keylist:
        short_key = long_short_key_map.get(key)
        if (key in list(kwargs.keys())):
            param_dict[key] = kwargs[key]
        elif key in param_dict:
            continue
        elif short_key and (short_key in list(kwargs.keys())):
            param_dict[key] = kwargs[short_key]

        elif len(arglist) >= 1:
            value = arglist.pop()
            if value != None:
                param_dict[key] = value
        elif key not in optional_keylist:
            LOGGER.error(" Invalid Input args %s is not in optional list %s" % (
                key, str(mandatory_options_list)))
            raise ValueError("Invalid Input Arguments")

        if key == 'con':
            if key in param_dict:
                con = param_dict.pop(key)

    # if not con:
    #    raise AttributeError("Invalid command invocation: Connection Object missing.")

    me_key_found = False
    for me_key in list(param_dict.keys()):
        # Checking mandatory option_list presence
        if me_key in list(mandatory_options_list.keys()):
            if not set(mandatory_options_list[me_key]).issubset(set(param_dict.keys())):
                LOGGER.error(" Invalid command invocation %s of mandatory list %s is not in arguments parovided" % (
                    me_key, str(mandatory_options_list)))
                raise AttributeError("Invalid command invocation")

        # Checking mutually exclusive key presense
        if me_key in mutually_exclusive_keys:
            if me_key_found:
                LOGGER.error(" Invalid command invocation %s of mutual exclusive list %s " % (
                    me_key, str(mutually_exclusive_keys)))
                raise AttributeError("Invalid command invocation")
            me_key_found = True

    if not set(keylist + list(long_short_key_map.values())).issuperset(set(kwargs.keys())):
        LOGGER.error(" Invalid Input args: %s unsupported argument passed"
                     % list(set(kwargs.keys()).difference(set(keylist + long_short_key_map.values()))))
        raise ValueError("Invalid Input Arguments")

    return con


def osimages(*args, **kwargs):
    '''
    @summary:
        Use this function to retrieve information about, delete, and import OS images,
         OS-image profiles, device driver, and boot-options files.
        data_dictionary = osimages(input_args, key=values )

        Where KeyList is as follows

            keylist = [con, subcmd, o]

    @param
        subcmd
            list                Retrieve information about all osimages
            globalsettings      Retrieve or modify global operating-system deployment
                                settings. Global settings serve as defaults settings
                                when operating systems are deployed.
            hostsettings        Retrieve information about the network and storage
                                settings for all servers, and create or modify the
                                network and storage settings for one or more servers
            hostplatforms       Retrieve information about the host platforms and
                                deploy operating-system images to the host platforms
                                as a job
            import              Import OS images and scripts from remote server to
                                LXCA
            remotefileservers   Retrieve information about all remote file-server
                                profiles or to create or modify a remote file-server
                                profile
            delete              Delete osimages from LXCA

    @example
        list all osimages info
        osimages(con, subcmd='list')

        delete osimages with ids
        osimages(con, subcmd='delete', id='i1,i2')
        rep = osimages(con_lxca, 'delete', i='20190131054310_trail.py')
        List all globalsettings
        osimages(con, subcmd = 'globalsettings')

        Set Linux default passw0rd using globalsettings
        change_linux_password = {
	"activeDirectory": {
		"allDomains": [],
		"defaultDomain": "labs.lenovo.com"
	},
	"credentials": [{
			"name": "root",
			"password": "Test1234",
			"passwordChanged": True,
			"type": "LINUX"
		}, {
			"type": "WINDOWS",
			"name": "Administrator",
			"password": None,
			"passwordChanged": False
		}
	],
	"ipAssignment": "dhcpv4",
	"isVLANMode": False,
	"licenseKeys": {
		"win2012r1": {
			"dataCenterLicenseKey": "",
			"standardLicenseKey": "",
		},
		"win2012r2": {
			"dataCenterLicenseKey": "",
			"standardLicenseKey": ""
		},
		"win2016r1": {
			"dataCenterLicenseKey": "",
			"standardLicenseKey": ""
		},
		"win2019r1": {
			"dataCenterLicenseKey": "",
			"standardLicenseKey": ""
		}
	}
}

json_string = json.dumps(change_linux_password)
print json_string

rep = osimages(con_lxca, subcmd = 'globalsettings', osimages_dict = json_string)


    Remote file server list
    rep = osimages(con_lxca, subcmd = 'remotefileservers')

    Create remote file server entry for ftp server

    rep = osimages(con_lxca, subcmd = 'remotefileservers',
			   osimages_dict ='{"username":"guest", "password":"Passw0rd",
			    "protocol":"FTP", "port": 21, "address":"10.243.2.207", "displayName": "new_ftp_207" }')

    Update remote file server
    rep = osimages(con_lxca, subcmd = 'remotefileservers',
                   osimages_dict ='{"putid": "1", "protocol":"FTP", "port": 21,
                    "address":"10.243.2.207", "displayName": "new_ftp_207" }')

    Delete remote file server
    rep = osimages(con_lxca, subcmd = 'remotefileservers', osimages_dict ='{"deleteid": "1"}')


    Import local files of imagetype (UNATTEND, CUSTOM_CONFIG, SCRIPT, OS)
    rep = osimages(con_lxca, subcmd='import', imagetype='UNATTEND')
    print rep
    file_dict = { "jobId":rep["jobId"], "imageName":"SLES", "os":"sles", "description":"SLES_config_file", "file": "/home/naval/sles_unattended.xml" }
    rep = osimages(con_lxca, subcmd='import', imagetype='UNATTEND', osimages_dict = json.dumps(file_dict))

    import BUNDLE and BUNDLESIG done with single jobid and imagename should be same as basename of files
    rep = osimages(con_lxca, subcmd='import', imagetype='BUNDLE')
    file_dict = {"jobId":rep["jobId"], "imageName":"bundle_win2016_20180926153236.zip", "file": "/home/naval/osimage_test/bundle_win2016_20180926153236.zip"}
    rep1 = osimages(con_lxca, subcmd='import', imagetype='BUNDLE', osimages_dict = json.dumps(file_dict))
    file_dict = { "jobId":rep["jobId"], "imageName":"bundle_win2016_20180926153236.zip.asc", "file": "/home/naval/osimage_test/bundle_win2016_20180926153236.zip.asc"}
    rep2 = osimages(con_lxca, subcmd='import', imagetype='BUNDLESIG', osimages_dict = json.dumps(file_dict))

    get all hostSettings
    rep = osimages(con_lxca, 'hostsettings')

    create hostsettings entry
    host_settings_dict = {u'hosts': [{u'storageSettings': {u'targetDevice': u'localdisk'}, u'uuid': u'A1445C6FDBAA11E6A87F86E06E3AFFFF', u'networkSettings': {u'dns2': u'', u'dns1': u'10.240.0.10', u'hostname': u'nodeundefined', u'vlanId': 0, u'selectedMAC': u'AUTO', u'gateway': u'10.243.0.1', u'subnetMask': u'255.255.240.0', u'mtu': 1500, u'prefixLength': 64, u'ipAddress': u'10.243.9.79'}}, {u'storageSettings': {u'targetDevice': u'localdisk'}, u'uuid': u'A122FB03FF4011E68D9BA32E3A66DDDD', u'networkSettings': {u'dns2': u'', u'dns1': u'10.240.0.10', u'hostname': u'proton1', u'vlanId': 0, u'selectedMAC': u'AUTO', u'gateway': u'10.243.0.1', u'subnetMask': u'255.255.240.0', u'mtu': 1500, u'prefixLength': 64, u'ipAddress': u'10.243.9.87'}}]}
    host_settings_json = json.dumps(host_settings_dict)
    rep = osimages(con_lxca, 'hostsettings', action='create', osimages_dict = host_settings_json)

    update hostSettings entry
    host_settings_dict = {u'hosts': [{u'storageSettings': {u'targetDevice': u'localdisk'}, u'uuid': u'A1445C6FDBAA11E6A87F86E06E3AFFFF', u'networkSettings': {u'dns2': u'', u'dns1': u'10.240.0.10', u'hostname': u'nodeundefined', u'vlanId': 0, u'selectedMAC': u'AUTO', u'gateway': u'10.243.0.1', u'subnetMask': u'255.255.240.0', u'mtu': 1500, u'prefixLength': 64, u'ipAddress': u'10.243.25.25'}}, {u'storageSettings': {u'targetDevice': u'localdisk'}, u'uuid': u'A122FB03FF4011E68D9BA32E3A66DDDD', u'networkSettings': {u'dns2': u'', u'dns1': u'10.240.0.10', u'hostname': u'proton1', u'vlanId': 0, u'selectedMAC': u'AUTO', u'gateway': u'10.243.0.1', u'subnetMask': u'255.255.240.0', u'mtu': 1500, u'prefixLength': 64, u'ipAddress': u'10.243.26.26'}}]}
    host_settings_json = json.dumps(host_settings_dict)
    rep = osimages(con_lxca, 'hostsettings', action='update', osimages_dict = host_settings_json)

    '''

    global SHELL_OBJ
    # #con = None
    # param_dict = {}
    # command_name = sys._getframe().f_code.co_name
    #
    # # con = kwargs.get('con')
    # # if not con:
    # #     raise ValueError("Invalid Input Arguments")
    #
    # logger.info(" osimages got kwargs %s " % str(kwargs))
    # if args:
    #     kwargs['osimages_info'] = args[0]
    # #param_dict = (args, kwargs)
    # logger.info(" osimages got param_dict %s " % str(kwargs))
    # # handle_input_dict only takes param_dict as input argument
    # ch = shell_obj.handle_input_dict(command_name, con, kwargs)
    #  return ch

    con = None
    param_dict = {}
    param_dict = kwargs
    kwargs = {}     # this is required  to avoid invalid argument error in _validate_param
    command_name = sys._getframe().f_code.co_name

    long_short_key_map = {}
    keylist = ['con', 'subcmd']
    optional_keylist = ['con']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def managementserver(*args, **kwargs):
    '''

@summary:
    Use this function to get repository info from Lenovo XClarity Administrator
    run this function as

    data_dictionary = managementserver( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con', 'subcmd', 'key', 'fixids', 'type', 'action', 'files','jobid']

@param
    The parameters for this command are as follows
    subcmd
        query               Retrieve information about all updates in the
                            management-server updates repository
        query_fixids        Retrieve information or the readme or change history
                            file for a specific update in the management-server
                            updates repository
        apply               install a management-server update.
        acquire             Downloads the specified firmware updates from Lenovo
                            XClarity Support website, and stores the updates to
                            the updates repository
        refresh             Retrieves information about the latest available
                            firmware updates from the Lenovo XClarity Support
                            website, and stores the information to the updates
                            repository
        delete              Deletes the specified fixids - removeMetadata not supported
        import              Import files to management server

    key    Returns the specified type of update. This can be one of the following values.
                all. Returns all information. This is the default value.
                currentVersion. Returns the current version of Lenovo XClarity Administrator.
                history. Returns the history of management-server updates.
                importDir. Returns the directory for the management-server updates repository.
                size. Returns the repository size (in bytes).
                updates. Returns information about all updates packages.
                updatedDate. Returns the date when the last update was performed.

     fixids    comma separated fixids
     type      Type for Get with fixids
                changeHistory. Returns the change-history file for the specified management-server update.
                readme. Returns the readme file for the specified management-server update
     jobid     jobid for import
     files     files to be imported with fullpath and comma separated
@example
    TO import files
    rep = managementserver(con_lxca, subcmd='import', files='/home/naval/updates/updates/lnvgy_sw_lxca_thinksystemrepo1-1.3.2_anyos_noarch.txt')
    rep = managementserver(con_lxca, subcmd='import', j=rep['jobid'], files='/home/naval/updates/updates/lnvgy_sw_lxca_thinksystemrepo1-1.3.2_anyos_noarch.txt')


    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None

    # some paramters don't have short options
    long_short_key_map = {'key': 'k', 'fixids': 'f',
                          'type': 't', 'jobid': 'j'}

    keylist = ['con', 'subcmd', 'key', 'fixids', 'type', 'files', 'jobid']
    optional_keylist = ['con', 'key', 'fixids',
                        'type', 'files', 'jobid']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)

    #remove dummy field added for view
    if 'dummy' in out_obj:
        out_obj.pop('dummy')
    return out_obj


def rules(*args, **kwargs):
    '''

@summary:
    Use this function to get and set complaince rules on Lenovo XClarity Administrator
    run this function as

    data_dictionary = managementserver( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con', 'id', 'rule']

@param
    The parameters for this command are as follows


@example

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None

    # some paramters don't have short options
    long_short_key_map = {'id': 'i', 'rule': 'r'}

    keylist = ['con', 'id', 'rule']
    optional_keylist = ['con', 'id', 'rule']
    mutually_exclusive_keys = ['id', 'rule']
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict, False)
    return out_obj


def compositeResults(*args, **kwargs):
    '''

@summary:
    Use this function to get and set complaince rules on Lenovo XClarity Administrator
    run this function as

    data_dictionary = managementserver( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con', 'id', ,'query_solutionGroups', 'solutionGroups','targetResources','all_rules']

@param
    The parameters for this command are as follows


@example

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None

    # some paramters don't have short options
    long_short_key_map = {'id': 'i', 'query_solutionGroups': 'q',
                          'solutionGroups': 's', 'targetResources': 't', 'all_rules': 'a'}

    keylist = ['con', 'id', 'query_solutionGroups',
               'solutionGroups', 'targetResources', 'all_rules']
    optional_keylist = ['con', 'id', 'query_solutionGroups',
                        'solutionGroups', 'targetResources', 'all_rules']
    mutually_exclusive_keys = ['id', 'query_solutionGroups',
                               'solutionGroups', 'targetResources', 'all_rules']
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict, False)
    return out_obj


def storedcredentials(*args, **kwargs):
    '''

@summary:
    Use this function to get and set complaince rules on Lenovo XClarity Administrator
    run this function as

    data_dictionary = storedcredentials( key1 = 'val1', key2 = 'val2', ...)

    Where KeyList is as follows

        keylist = ['con', 'id', 'user_name', 'description', 'password', 'delete_id']

@param
    The parameters for this command are as follows
    id          Stored credential id of stored credential
    user_name    user name
    password    password
    description description of user credential
    delete_id    id to be deleted

@example
    rep = storedcredentials(con1)
    rep = storedcredentials(con1, id='955')
    rep = storedcredentials(con1,u='admin1', p='admin1', d='description of stored credentials')
    rep = storedcredentials(con1, delete_id='954')
    rep = storedcredentials(con1,i='955', u='admin1', p='admin1', d='description of stored credentials for admin')

    '''
    global SHELL_OBJ
    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None

    # some paramters don't have short options
    long_short_key_map = {'id': 'i', 'user_name': 'u',
                          'description': 'd', 'password': 'p'}

    keylist = ['con', 'id', 'user_name',
               'description', 'password', 'delete_id']
    optional_keylist = ['con', 'id', 'user_name',
                        'description', 'password', 'delete_id']
    mutually_exclusive_keys = ['id', 'delete_id']
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict)
    return out_obj


def license(*args, **kwargs):
    '''

@summary:
    Use this function to retrieve information about warnings regarding non-compliance of installed Lenovo XClarity Administrator license
    run this function as
    license(con)

@param
    The parameter for this command is as follows

        con      Connection Object to Lenovo XClarity Administrator

@example 
    license(con)
    '''
    global SHELL_OBJ

    command_name = sys._getframe().f_code.co_name
    param_dict = {}
    con = None

    long_short_key_map = {}
    keylist = ['con']
    optional_keylist = ['con']
    mutually_exclusive_keys = []
    mandatory_options_list = {}

    con = _validate_param(keylist, long_short_key_map, mandatory_options_list, optional_keylist,
                          mutually_exclusive_keys,
                          param_dict, *args, **kwargs)

    out_obj = SHELL_OBJ.handle_input_dict(command_name, con, param_dict, False)
    return out_obj




'''
@since: 4 Sep 2015
@author: Prashant Bhosale <pbhosale@lenovo.com>
@license: Lenovo License
@copyright: Copyright 2016, Lenovo
@organization: Lenovo
@summary: This module is for creating a connection session object for given xHMC.

'''

import requests
import logging, json
import os, platform
import base64
import pkg_resources
from _socket import timeout
from requests.sessions import session
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.poolmanager import PoolManager

logger = logging.getLogger(__name__)

class lxcaAdapter(HTTPAdapter):

    def init_poolmanager(self, connections, maxsize, block=False):

        self.poolmanager = PoolManager(num_pools=connections,

                                       maxsize=maxsize,

                                       block=block)


class Error(Exception):
    """This exception is raised for any other low level errors """
    pass

class ConnectionError(Error):
    """This exception is raised when a connection related problem occurs, where a retry might make sense."""
    pass

class lxca_connection(object):
    '''
    C
    '''
    def __init__(self, url, user = None,  passwd = None, verify_callback = True, retries = 3):
        self.url = url
        self.user = user
        self.passwd = base64.b16encode(passwd.encode())
        self.retires = retries
        self.debug = False
        self.session = None
        #self.verify_callback = verify_callback
        #os.environ['REQUESTS_CA_BUNDLE'] = os.path.join('/etc/ssl/certs/','ca-certificates.crt')

        if verify_callback:
            self.verify_callback = os.environ['REQUESTS_CA_BUNDLE']
        else:
            self.verify_callback = verify_callback

    def __repr__(self):
        return "%s(%s, %s, debug=%s)" %(self.__class__.__name__, repr(self.url), self.user, repr(self.debug))

    def connect(self):
        '''
        connection function
        '''
        try:
            logger.debug("Establishing Connection")
            self.session = requests.session()
            self.session.verify = self.verify_callback

            pylxca_version = pkg_resources.require("pylxca")[0].version
            # Update the headers with your custom ones
            # You don't have to worry about case-sensitivity with
            # the dictionary keys, because default_headers uses a custom
            # CaseInsensitiveDict implementation within requests' source code.
            self.session.headers.update({'content-type': 'application/json; charset=utf-8','User-Agent': 'LXCA via Python Client / ' + pylxca_version})


            payload = dict(UserId= self.user, password=base64.b16decode(self.passwd).decode())
            pURL = self.url + '/sessions'
            self.session.mount(self.url, lxcaAdapter(max_retries=self.retires))
            r = self.session.post(pURL,data = json.dumps(payload),headers=dict(Referer=pURL),verify=self.verify_callback, timeout = 3)
            r.raise_for_status()
        except ConnectionError as e:
            logger.debug("Connection Exception as ConnectionError: Exception = %s", e)
            return False
        except requests.exceptions.HTTPError as e:
            logger.debug("Connection Exception as HttpError: Exception = %s", e.response.text)
            return False
        except Exception as e:
            logger.debug("Connection Exception: Exception = %s", e)
            return False

        '''
        Even though the csrf-token cookie will be automatically sent with the request,
        the server will be still expecting a valid X-Csrf-Token header,
        So we need to set it explicitly here
        '''
        if r.status_code == requests.codes['ok']:
            self.session.headers.update({'X-Csrf-Token': self.session.cookies.get('csrf')})

        return  True

    def test_connection(self):
        '''
        Test Connection from requests module
        '''
        try:
            test_url = self.url + '/aicc'
            resp = self.session.get(test_url,verify=self.session.verify, timeout=3)
            #If valid JSON object is parsed then the connection is successfull
            py_obj = resp.json()
        except Exception as e:
            raise ConnectionError("Invalid connection")
        return

    def get_url(self):
        return self.url

    def get_session(self):
        return self.session

    def disconnect(self):
        '''
        session Disconnection
        '''
        result = False
        try:
            delete_url = self.url + '/sessions'
            resp = self.session.delete(delete_url, verify=False, timeout=3)
            py_obj = resp.json()
            logger.debug("Deleted session on lxca = %s", py_obj)
            result = True
        except Exception as e:
            logger.debug("Unable to delete session on lxca = %s", e)
            raise ConnectionError("Invalid connection: Connection is not Initialized")

        self.url = None
        self.user = None
        self.passwd = None
        self.debug = False
        try:
            self.session.close()
        except Exception as e:
            logger.debug("Connection with invalid session = %s", e)
            raise Exception("Invalid connection: Connection is not Initialized")
        self.session = None
        return result

    def ping(self, host):
        """
        Returns True if host responds to a ping request
        """

        # Ping parameters as function of OS
        ping_str = "-n 1" if  platform.system().lower()=="windows" else "-c 1"

        # Ping
        return os.system("ping " + ping_str + " " + host) == 0



'''
@since: 21 Oct 2015
@author: Prashant Bhosale <pbhosale@lenovo.com>
@license: Lenovo License
@copyright: Copyright 2016, Lenovo
@organization: Lenovo
@summary: This module implement facade interface for pylxca API interface. It provides a 
single entry point for APIs.   
'''


import logging.config
import json

from pylxca.pylxca_api.lxca_connection import lxca_connection
from pylxca.pylxca_api.lxca_connection import ConnectionError
from  pylxca.pylxca_api.lxca_rest import lxca_rest
from  pylxca.pylxca_api.lxca_rest import HTTPError

logger = logging.getLogger(__name__)

class Singleton(type):
    
    def __call__(cls, *args, **kwargs):#@NoSelf
        try:
            return cls.__instance
        except AttributeError:
            cls.__instance = super(Singleton, cls).__call__(*args, **kwargs)
            return cls.__instance

def with_metaclass(meta, *bases):
    class metaclass(meta):
        __call__ = type.__call__
        __init__ = type.__init__
        def __new__(cls, name, this_bases, d):
            if this_bases is None:
                return type.__new__(cls, name, (), d)
            return meta(name, bases, d)
    return metaclass('temporary_class', None, {})

class lxca_api(with_metaclass(Singleton, object)):
    '''
    Facade class which exposes interface for accessing various LXCA APIs
    '''
    
    def __init__(self):
        '''
        Constructor
        '''
        
        self.con = None
        self.func_dict = {'connect':self.connect,
                          'disconnect':self.disconnect,
                          'chassis':self.get_chassis,
                          'nodes':self.get_nodes,
                          'switches':self.get_switches,
                          'fans':self.get_fans,
                          'powersupplies':self.get_powersupply,
                          'fanmuxes':self.get_fanmux,
                          'cmms':self.get_cmm,
                          'scalablesystem':self.get_scalablesystem,
                          'log':self.get_log_level,
                          'discover':self.do_discovery,
                          'manage':self.do_manage,
                          'unmanage':self.do_unmanage,
                          'configpatterns':self.do_configpatterns,
                          'configprofiles':self.get_configprofiles,
                          'configtargets':self.get_configtargets,
                          'updatepolicy':self.do_updatepolicy,
                          'updaterepo':self.get_updaterepo,
                          'updatecomp':self.do_updatecomp,
                          'managementserver':self.do_managementserver,
                          'users':self.get_users,
                          'ffdc':self.get_ffdc,
                          'jobs':self.get_jobs,
                          'lxcalog':self.get_lxcalog,
                          'tasks':self.get_set_tasks,
                          'manifests':self.get_set_manifests,
                          'osimages':self.get_set_osimage,
                          'resourcegroups':self.get_set_resourcegroups,
                          'rules': self.get_set_rules,
                          'compositeResults': self.get_set_compositeResults,
                          'storedcredentials': self.get_set_storedcredentials,
                          'license': self.get_license
                        }
    
    def api( self, object_name, dict_handler = None, con = None ):
        
 
        try:
            # If Any connection is establibshed
            if con == None and self.con and isinstance(self.con,lxca_connection):
                con = self.con

            if object_name  == "disconnect":
                dict_handler['orig_con'] = self.con

            if object_name  != "connect":
                if con and isinstance(con,lxca_connection): 
                    self.con = con
                else:
                    raise ConnectionError("Invalid Connection Object")


            return self.func_dict[object_name](dict_handler)
        except ConnectionError as re:
            logger.error("Connection Exception: Exception = %s", re)
            if self.con: 
                self.con.disconnect()
                self.con = None
            raise re
        except HTTPError as re:
            logger.error("Exception %s Occurred while calling REST API for object %s" %(re, object_name))
            raise re
        except ValueError as re:
            logger.error("Exception ValueError %s Occurred while calling REST API for object %s" %(re, object_name))
            raise re
        except AttributeError as re:
            logger.error("Exception AttributeError %s Occurred while calling REST API for object %s" %(re, object_name))
            raise re
        except Exception as re:
            logger.error("Exception: %s  %s Occurred while calling REST API for object %s" %(type(re), str(re), object_name))
            raise re
        return None
    
    def connect( self, dict_handler = None ):
        url = user = passwd = None
        verify = True
        if dict_handler:
            url = next(item for item in [dict_handler.get('l') , dict_handler.get('url')] if item is not None)
            user = next(item for item in [dict_handler.get('u') , dict_handler.get('user')] if item is not None)
            passwd = next(item for item in [dict_handler.get('p') , dict_handler.get('pw')] if item is not None)
            if "noverify" in dict_handler: verify = False
                 
        self.con = lxca_connection(url,user,passwd,verify)
        
        #clear the temp stored passwd
        passwd = None
        
        if self.con.connect() == True:
            self.con.test_connection()
            logger.debug("Connection to LXCA Success")
            return self.con    
        else:
            logger.error("Connection to LXCA Failed")
            self.con = None
            return self.con
    
    def disconnect( self, dict_handler=None):
        """
            this method perform disconnect operation
             it also reset current connection to original connection this is used in api version
             to retain origianal connection if we are disconnecting other than current connection

             i.e
             con1 = connect(...)
             con2 = connect(...)
             con3 = connect(...)
             con4 = connect(...)

             disconnect(con2)  will keep current connection to con4
             disconnect(con4) or disconnect() will set current connection to None

        :param dict_handler:  orig_con have original connection before call of disconnect()
        :return:
        """
        resp = False
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")

        reset_conn_to_orig = False
        if dict_handler and ("orig_con" in dict_handler):
            if self.con !=  dict_handler['orig_con']:
                reset_conn_to_orig = True
        try:
            resp = self.con.disconnect()
        except Exception as e:
            if reset_conn_to_orig:
                self.con = dict_handler['orig_con']
            raise
        if reset_conn_to_orig:
            self.con = dict_handler['orig_con']
        else:
            self.con = None
        return resp

    def get_log_level(self, dict_handler=None):
        lvl = None
        if dict_handler:
            lvl = next((item for item in [dict_handler.get('l') , dict_handler.get('lvl')] if item is not None),None)
        if lvl == None:
            lvl = lxca_rest().get_log_level()
        else:
            return self.set_log_level(lvl)
        return lvl
    
    def set_log_level(self, log_value):
        try:
            lxca_rest().set_log_level(log_value)
            logger.debug("Current Log Level is now set to " + str(logger.getEffectiveLevel()))
        except Exception as e:
            logger.error("Fail to set Log Level")
            return False
        return True

    def get_chassis( self, dict_handler = None ):
        uuid = None
        status = None
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            uuid = next((item for item in [dict_handler.get('u') , dict_handler.get('uuid')] if item is not None),None)
            status = next((item for item in [dict_handler.get('s') , dict_handler.get('status')] if item is not None),None)
        
        resp = lxca_rest().get_chassis(self.con.get_url(),self.con.get_session(),uuid,status)
        py_obj = json.loads(resp.text)
        return py_obj
    
    def get_nodes( self, dict_handler = None ):
        uuid = None
        status = None
        chassis_uuid = None
        
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            uuid = next((item for item in [dict_handler.get('u') , dict_handler.get('uuid')] if item is not None),None)
            modify = next((item for item in [dict_handler.get('m'), dict_handler.get('modify')] if item is not None), None)
            status = next((item for item in [dict_handler.get('s') , dict_handler.get('status')] if item is not None),None)
            chassis_uuid = next((item for item in [dict_handler.get('c') , dict_handler.get('chassis')] if item is not None),None)
            metrics = next((True for item in [dict_handler.get('x', False), dict_handler.get('metrics', False)] if item is None or item is True), False)
            
        if chassis_uuid:
            resp = lxca_rest().get_chassis(self.con.get_url(),self.con.get_session(),chassis_uuid,status)
            py_obj = json.loads(resp.text)
            py_obj = {'nodesList':py_obj["nodes"]}

        if uuid and modify:
            resp = lxca_rest().set_nodes(self.con.get_url(), self.con.get_session(), uuid, modify)
            return resp

        if metrics:
            resp = lxca_rest().get_metrics(self.con.get_url(), self.con.get_session(), uuid)
            py_obj = json.loads(resp.text)
        else:
            resp = lxca_rest().get_nodes(self.con.get_url(),self.con.get_session(),uuid,status)
            py_obj = json.loads(resp.text)
        return py_obj
        
    def get_switches( self, dict_handler = None ):
        uuid = None
        chassis_uuid = None
        port_name = None
        list_port = None
        action = None

        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            uuid = next((item for item in [dict_handler.get('u') , dict_handler.get('uuid')] if item is not None),None)
            chassis_uuid = next((item for item in [dict_handler.get('c') , dict_handler.get('chassis')] if item is not None),None)
            port_name = next((item for item in [dict_handler.get('ports')] if item is not None),None)
            action = next((item for item in [dict_handler.get('action')] if item is not None),
                             None)
            #if "ports" in dict_handler: list_port = True
            if port_name: list_port = True

        if chassis_uuid:
            resp = lxca_rest().get_chassis(self.con.get_url(),self.con.get_session(),chassis_uuid,None)
            py_obj = json.loads(resp.text)
            py_obj = {'switchesList':py_obj["switches"]}
        elif list_port and (action==None):
            resp = lxca_rest().get_switches_port(self.con.get_url(), self.con.get_session(), uuid, list_port)
            py_obj = json.loads(resp.text)
        elif port_name and action:
            resp = lxca_rest().put_switches_port(self.con.get_url(), self.con.get_session(), uuid, port_name, action)
            py_obj = json.loads(resp.text)
        else:
            resp = lxca_rest().get_switches(self.con.get_url(),self.con.get_session(),uuid)
            py_obj = json.loads(resp.text)
        return py_obj
    
    def get_fans( self, dict_handler = None ):
        uuid = None
        chassis_uuid = None
        
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            uuid = next((item for item in [dict_handler.get('u') , dict_handler.get('uuid')] if item is not None),None)
            chassis_uuid = next((item for item in [dict_handler.get('c') , dict_handler.get('chassis')] if item is not None),None)
            
        if chassis_uuid:
            resp = lxca_rest().get_chassis(self.con.get_url(),self.con.get_session(),chassis_uuid,None)
            py_obj = json.loads(resp.text)
            py_obj = {'fansList':py_obj["fans"]}
        else:
            resp = lxca_rest().get_fan(self.con.get_url(),self.con.get_session(),uuid)
            py_obj = json.loads(resp.text)
        return py_obj

    def get_powersupply( self, dict_handler = None ):
        uuid = None
        chassis_uuid = None
        
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            uuid = next((item for item in [dict_handler.get('u') , dict_handler.get('uuid')] if item is not None),None)
            chassis_uuid = next((item for item in [dict_handler.get('c') , dict_handler.get('chassis')] if item is not None),None)
            
        if chassis_uuid:
            resp = lxca_rest().get_chassis(self.con.get_url(),self.con.get_session(),chassis_uuid,None)
            py_obj = json.loads(resp.text)
            py_obj = {'powersuppliesList':py_obj["powerSupplies"]}
        else:
            resp = lxca_rest().get_powersupply(self.con.get_url(),self.con.get_session(),uuid)
            py_obj = json.loads(resp.text)
        return py_obj
    
    def get_fanmux( self, dict_handler = None ):
        uuid = None
        chassis_uuid = None
        
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            uuid = next((item for item in [dict_handler.get('u') , dict_handler.get('uuid')] if item is not None),None)
            chassis_uuid = next((item for item in [dict_handler.get('c') , dict_handler.get('chassis')] if item is not None),None)
            
        if chassis_uuid:
            resp = lxca_rest().get_chassis(self.con.get_url(),self.con.get_session(),chassis_uuid,None)
            py_obj = json.loads(resp.text)
            py_obj = {'fanmuxesList':py_obj["fanMuxes"]}
        else:
            resp = lxca_rest().get_fanmux(self.con.get_url(),self.con.get_session(),uuid)
            py_obj = json.loads(resp.text)
        return py_obj

    def get_cmm( self, dict_handler = None ):
        uuid = None
        chassis_uuid = None
        
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            uuid = next((item for item in [dict_handler.get('u') , dict_handler.get('uuid')] if item is not None),None)
            chassis_uuid = next((item for item in [dict_handler.get('c') , dict_handler.get('chassis')] if item is not None),None)
            
        if chassis_uuid:
            resp = lxca_rest().get_chassis(self.con.get_url(),self.con.get_session(),chassis_uuid,None)
            py_obj = json.loads(resp.text)
            py_obj = {'cmmsList':py_obj["cmms"]}
        else:
            resp = lxca_rest().get_cmm(self.con.get_url(),self.con.get_session(),uuid)
            py_obj = json.loads(resp.text)
        return py_obj

    def get_scalablesystem( self, dict_handler = None ):
        complexid = None
        complextype = None

        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            complexid = next((item for item in [dict_handler.get('i') , dict_handler.get('id')] if item is not None),None)
            complextype = next((item for item in [dict_handler.get('t') , dict_handler.get('type')] if item is not None),None)

        resp = lxca_rest().get_scalablesystem(self.con.get_url(),self.con.get_session(),complexid,complextype)
        py_obj = json.loads(resp.text)
        return py_obj

    def do_discovery( self, dict_handler = None ):
        ip_addr = None
        jobid = None
        
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            ip_addr = next((item for item in [dict_handler.get  ('i') , dict_handler.get('ip')] if item is not None),None)
            jobid = next((item for item in [dict_handler.get  ('j') , dict_handler.get('job')] if item is not None),None)
        
        resp = lxca_rest().do_discovery(self.con.get_url(),self.con.get_session(),ip_addr,jobid)
        
        try:
            py_obj = resp
            return py_obj
        except AttributeError as ValueError:
            raise ValueError

    def do_manage( self, dict_handler = None ):
        ip_addr = None
        user = None
        pw = None
        rpw = None
        jobid = None
        force = None
        storedcredential_id = None
        
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            ip_addr = next((item for item in [dict_handler.get  ('i') , dict_handler.get('ip')] if item is not None),None)
            user = next((item for item in [dict_handler.get  ('u') , dict_handler.get('user')] if item is not None),None)
            pw = next((item for item in [dict_handler.get  ('p') , dict_handler.get('pw')] if item is not None),None)
            rpw = next((item for item in [dict_handler.get  ('r') , dict_handler.get('rpw')] if item is not None),None)
            jobid = next((item for item in [dict_handler.get  ('j') , dict_handler.get('job')] if item is not None),None)
            storedcredential_id = next((item for item in [dict_handler.get  ('s') ,
                                dict_handler.get('storedcredential_id')] if item is not None),None)
            force = next((item for item in [dict_handler.get  ('f') , dict_handler.get('force')] if item is not None),None)

        resp = lxca_rest().do_manage(self.con.get_url(),self.con.get_session(),ip_addr,user,
                                     pw,rpw,force,jobid, storedcredential_id)
        
        try:
            py_obj = resp
            return py_obj
        except AttributeError as ValueError:
            raise ValueError


    def do_unmanage( self, dict_handler = None ):
        endpoints = None
        force = None
        jobid = None
        
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            endpoints = next((item for item in [dict_handler.get  ('e') , dict_handler.get('ep')] if item is not None),None)
            force = next((item for item in [dict_handler.get  ('f') , dict_handler.get('force')] if item is not None),False)
            jobid = next((item for item in [dict_handler.get  ('j') , dict_handler.get('job')] if item is not None),None)
            
        resp = lxca_rest().do_unmanage(self.con.get_url(),self.con.get_session(),endpoints,force,jobid)
        
        try:
            py_obj = resp
        except AttributeError as ValueError:
            raise ValueError
        return py_obj
        
    def get_configtargets( self, dict_handler = None ):
        targetid = None

        if not self.con:
            raise ConnectionError("Connection is not Initialized.")

        if dict_handler:
            targetid = next((item for item in [dict_handler.get  ('i') , dict_handler.get('id')] if item is not None),None)

        resp = lxca_rest().get_configtargets(self.con.get_url(),self.con.get_session(),targetid)

        try:
            py_obj = json.loads(resp.text)
            return py_obj

        except AttributeError as ValueError:
            raise ValueError
    
    def do_configpatterns( self, dict_handler = None ):
        patternid = None
        patternname = None
        includeSettings = None
        endpoint = None
        restart = None
        etype = None
        pattern_update_dict = None
        subcmd = None

        if not self.con:
            raise ConnectionError("Connection is not Initialized.")

        try:

            if dict_handler:
                patternid = next((item for item in [dict_handler.get  ('i') , dict_handler.get('id')] if item is not None),None)
                patternname = next((item for item in [dict_handler.get('n'), dict_handler.get('name')] if item is not None),
                                None)
                includeSettings = next((item for item in [dict_handler.get('includeSettings')] if item is not None),None)
                endpoint = next((item for item in [dict_handler.get  ('e') , dict_handler.get('endpoint')] if item is not None),None)
                restart = next((item for item in [dict_handler.get  ('r') , dict_handler.get('restart')] if item is not None),None)
                etype = next((item for item in [dict_handler.get  ('t') , dict_handler.get('type')] if item is not None),None)
                pattern_update_dict = next((item for item in [dict_handler.get('pattern_update_dict')] if item is not None), None)
                subcmd = next((item for item in [dict_handler.get('subcmd')] if item is not None), None)
            if patternname and not patternid:
                # get all patterns and get id from name
                resp = lxca_rest().do_configpatterns(self.con.get_url(), self.con.get_session(), None, None,
                                                    None, None, None, None)
                py_obj = json.loads(resp.text)
                for item in py_obj['items']:
                    if item['name'] == patternname:
                        patternid = item['id']
                        break
                if not patternid:
                    raise Exception("Pattern Name %s not found" %patternname)
            if subcmd == 'status':
                resp = lxca_rest().get_configstatus(self.con.get_url(), self.con.get_session(), endpoint)
            else:
                resp = lxca_rest().do_configpatterns(self.con.get_url(),self.con.get_session(),patternid, includeSettings, endpoint, restart, etype, pattern_update_dict)

            # if endpoint:
            #     return resp
            # else:
            py_obj = json.loads(resp.text)
            return py_obj

        except HTTPError as e:
            logger.error(e)
            if e.response.status_code == 403:
                return e.response.json()
            else:
                logger.error("Exception %s Occurred while calling REST API" %(e))
                raise e

        except AttributeError as ValueError:
            raise ValueError
    
    def get_configprofiles( self, dict_handler = None ):
        profileid = None
        profilename = None
        endpoint = None
        restart = None
        subcmd = None
        powerdown = None
        resetimm = None
        resetswitch = None
        force = None

        if not self.con:
            raise ConnectionError("Connection is not Initialized.")

        try:

            if dict_handler:
                profileid = next((item for item in [dict_handler.get  ('i') , dict_handler.get('id')] if item is not None),None)
                profilename = next((item for item in [dict_handler.get('n'), dict_handler.get('name')] if item is not None),
                                None)
                endpoint = next(
                    (item for item in [dict_handler.get('e'), dict_handler.get('endpoint')] if item is not None), None)
                restart = next((item for item in [dict_handler.get('r'), dict_handler.get('restart')] if item is not None),
                            None)
                subcmd = next((item for item in [dict_handler.get('subcmd')] if item is not None),
                            None)
                powerdown = next((item for item in [dict_handler.get('p'), dict_handler.get('powerdown')] if item is not None),
                    None)
                resetimm = next(
                    (item for item in [dict_handler.get('resetimm')] if item is not None),
                    None)
                resetswitch = next(
                    (item for item in [dict_handler.get('resetswitch')] if item is not None),
                    None)

                force = next((item for item in [dict_handler.get('f'), dict_handler.get('force')] if item is not None),
                            None)

            if subcmd == 'rename' and profilename:
                resp = lxca_rest().put_configprofiles(self.con.get_url(), self.con.get_session(), profileid, profilename)
            elif subcmd == 'activate' and endpoint and restart:
                resp = lxca_rest().post_configprofiles(self.con.get_url(), self.con.get_session(), profileid, endpoint, restart)
            elif subcmd == 'delete' and profileid:
                resp = lxca_rest().delete_configprofiles(self.con.get_url(), self.con.get_session(), profileid)
            elif subcmd == 'unassign' and profileid:
                resp = lxca_rest().unassign_configprofiles(self.con.get_url(), self.con.get_session(), profileid, powerdown, resetimm, resetswitch, force)
                if len(resp.text):
                    py_obj = json.loads(resp.text)
                    py_obj['dummy'] = {'status': []}
                    return py_obj

            elif subcmd == 'list':
                resp = lxca_rest().get_configprofiles(self.con.get_url(),self.con.get_session(),profileid)

            if len(resp.text):
                py_obj = json.loads(resp.text)
                return py_obj
            elif resp.status_code == 204:   # Its success for rename of profile with empty text
                return { 'ID':profileid, 'name':profilename}

        except HTTPError as e:
            logger.error(e)
            if e.response.status_code == 403:
                return e.response.json()
            else:
                logger.error("Exception %s Occurred while calling REST API" %(e))
                raise e

        except AttributeError as ValueError:
            raise ValueError
    
    def get_jobs( self, dict_handler = None ):        
        jobid = None
        uuid = None
        state = None
        canceljobid = None
        deletejobid = None
        
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            jobid = next((item for item in [dict_handler.get  ('i') , dict_handler.get('id')] if item is not None),None)
            uuid = next((item for item in [dict_handler.get  ('u') , dict_handler.get('uuid')] if item is not None),False)
            state = next((item for item in [dict_handler.get  ('s') , dict_handler.get('state')] if item is not None),None)
            canceljobid = next((item for item in [dict_handler.get  ('c') , dict_handler.get('cancel')] if item is not None),None)
            deletejobid = next((item for item in [dict_handler.get  ('d') , dict_handler.get('delete')] if item is not None),None)
            
        resp = lxca_rest().get_jobs(self.con.get_url(),self.con.get_session(),jobid,uuid,state,canceljobid,deletejobid)
        
        try:
            if jobid:
                py_obj = json.loads(resp.text)
                py_obj = {'jobsList': [py_obj]}
                return py_obj
            if canceljobid or deletejobid:
                return resp
            else:
                py_obj = json.loads(resp.text)
                py_obj = {'jobsList':py_obj}
            return py_obj
        except AttributeError as ValueError:
            raise ValueError



    def get_users( self, dict_handler = None ):
        userid = None
        
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            userid = next((item for item in [dict_handler.get  ('i') , dict_handler.get('id')] if item is not None),None)
            
        resp = lxca_rest().get_users(self.con.get_url(),self.con.get_session(),userid)
        
        try:
            
            if userid:
                py_obj = json.loads(resp.text)['response']
            else:
                py_obj = json.loads(resp.text)
                py_obj = {'usersList':py_obj['response']}
        except AttributeError as ValueError:
            raise ValueError
        return py_obj
    
    def get_lxcalog( self, dict_handler = None ):        
        filter = None
        
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            filter = next((item for item in [dict_handler.get  ('f') , dict_handler.get('filter')] if item is not None),None)
            
        resp = lxca_rest().get_lxcalog(self.con.get_url(),self.con.get_session(),filter)
        
        try:
            py_obj = json.loads(resp.text)
            py_obj = {'eventList':py_obj}
            return py_obj
        except AttributeError as ValueError:
            raise ValueError
        
    def get_ffdc( self, dict_handler = None ): 
        uuid = None
        
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            uuid = next((item for item in [dict_handler.get  ('u') , dict_handler.get('uuid')] if item is not None),None)
            
        resp = lxca_rest().get_ffdc(self.con.get_url(),self.con.get_session(),uuid)
        
        try:
            py_obj = resp
        except AttributeError as ValueError:
            raise ValueError
        return py_obj
    
    
    def do_updatepolicy( self, dict_handler = None ):
        info = None
        policy = None
        jobid= None

        if not self.con:
            raise ConnectionError("Connection is not Initialized.")

        if dict_handler:
            policy = next((item for item in [dict_handler.get('p'), dict_handler.get('policy')] if item is not None),
                          None)
            info = next((item for item in [dict_handler.get('i'), dict_handler.get('info')] if item is not None), None)
            uuid = next((item for item in [dict_handler.get('u'), dict_handler.get('uuid')] if item is not None), None)
            jobid = next((item for item in [dict_handler.get('j'), dict_handler.get('jobid')] if item is not None),
                         None)
            type = next((item for item in [dict_handler.get('t'), dict_handler.get('type')] if item is not None),
                         None)

        if policy:
            resp = lxca_rest().post_updatepolicy(self.con.get_url(), self.con.get_session(), policy, type, uuid)
        else:
            resp = lxca_rest().get_updatepolicy(self.con.get_url(), self.con.get_session(), info, jobid, uuid)

        try:
            py_obj = json.loads(resp.text)
            if info == "RESULTS":
                py_obj = py_obj["all"]
        except AttributeError as ValueError:
            raise ValueError
        return py_obj

    def get_updaterepo( self, dict_handler = None ):
        key = None
        action = None
        mt = None
        scope = None
        fixids = None
        type = None
        
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            key = next((item for item in [dict_handler.get('k'), dict_handler.get('key')] if item is not None), None)
            action = next((item for item in [dict_handler.get('subcmd')] if item is not None), None)
            mt = next((item for item in [dict_handler.get('m'), dict_handler.get('mt')] if item is not None), None)
            scope = next((item for item in [dict_handler.get('s'), dict_handler.get('scope')] if item is not None), None)
            fixids = next((item for item in [dict_handler.get('f'), dict_handler.get('fixids')] if item is not None), None)
            type = next((item for item in [dict_handler.get('t'), dict_handler.get('type')] if item is not None), None)

        if key:
            resp = lxca_rest().get_updaterepo(self.con.get_url(),self.con.get_session(), key, mt, scope)
        elif action:
            resp = lxca_rest().put_updaterepo(self.con.get_url(), self.con.get_session(), action, fixids, mt, type, scope)
        else:
            raise Exception("Invalid argument")

        try:
            py_obj = json.loads(resp.text)
        except AttributeError as ValueError:
            raise ValueError
        return py_obj



    def do_managementserver(self, dict_handler=None):
        key = None
        action = None
        fixids = None
        type = None
        jobid = None
        files = None

        if not self.con:
            raise ConnectionError("Connection is not Initialized.")

        if dict_handler:
            key = next((item for item in [dict_handler.get('k'), dict_handler.get('key')] if item is not None), None)
            action = next((item for item in [dict_handler.get('subcmd')] if item is not None),
                          None)
            fixids = next((item for item in [dict_handler.get('f'), dict_handler.get('fixids')] if item is not None),
                          None)
            type = next((item for item in [dict_handler.get('t'), dict_handler.get('type')] if item is not None), None)
            jobid = next((item for item in [dict_handler.get('j'), dict_handler.get('jobid')] if item is not None), None)
            files = next((item for item in [dict_handler.get('files')] if item is not None),       None)
        if key or type:
            resp = lxca_rest().get_managementserver(self.con.get_url(), self.con.get_session(), key, fixids, type)
        elif action:
            resp = lxca_rest().set_managementserver(self.con.get_url(), self.con.get_session(), action, files, jobid, fixids)

        try:
            py_obj = json.loads(resp.text)
        except AttributeError as ValueError:
            raise ValueError
        py_obj['dummy']={'status':[]}
        return py_obj

    def do_updatecomp( self, dict_handler = None ):
        mode = None
        action = None
        server = None
        switch = None
        cmm = None
        storage = None
        query = None
        dev_list = None
                
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            mode = next((item for item in [dict_handler.get  ('m') , dict_handler.get('mode')] if item is not None),None)
            action = next((item for item in [dict_handler.get  ('a') , dict_handler.get('action')] if item is not None),None)
            server = next((item for item in [dict_handler.get  ('s') , dict_handler.get('server')] if item is not None),None)
            storage = next((item for item in [dict_handler.get  ('t') , dict_handler.get('storage')] if item is not None),None)
            switch = next((item for item in [dict_handler.get  ('w') , dict_handler.get('switch')] if item is not None),None)
            cmm = next((item for item in [dict_handler.get  ('c') , dict_handler.get('cmm')] if item is not None),None)
            query = next((item for item in [dict_handler.get('q'), dict_handler.get('query')] if item is not None), None)
            dev_list = next((item for item in [dict_handler.get('l'), dict_handler.get('dev_list')] if item is not None), None)

        resp = lxca_rest().do_updatecomp(self.con.get_url(),self.con.get_session(), query, mode,action,server,switch,storage,cmm, dev_list)
        
        try:
            if mode == None and action == None and server == None and  switch == None and storage == None and cmm == None and dev_list == None :
                py_obj = json.loads(resp.text)
            else:
                py_obj = resp.json()
            return py_obj
        except AttributeError as ValueError:
            raise ValueError
        return py_obj

    def get_set_tasks(self, dict_handler=None):
        job_uuid = None
        includeChildren = False
        action = None
        status = None
        cmd_updateList = None
        template = None

        if not self.con:
            raise ConnectionError("Connection is not Initialized.")

        if dict_handler:
            job_uuid = next((item for item in [ dict_handler.get('jobUID')] if item is not None), None)
            includeChildren = next((item for item in [ dict_handler.get('children')] if item is not None), "true")
            action = next((item for item in [dict_handler.get('action')] if item is not None), None)
            updateList = next((item for item in [dict_handler.get('updateList')] if item is not None), None)
            template = next((item for item in [dict_handler.get('template')] if item is not None), None)
            #if updateList:
            #    updateList = updateList['taskList']


        if job_uuid and action in ['cancel']:
            resp = lxca_rest().put_tasks(self.con.get_url(), self.con.get_session(), job_uuid, action)
            py_obj = resp.status_code
        elif action in ['delete']:
            resp = lxca_rest().delete_tasks(self.con.get_url(), self.con.get_session(), job_uuid)
            py_obj = resp.status_code
        elif action in ['update']:
            resp = lxca_rest().put_tasks_update(self.con.get_url(), self.con.get_session(), updateList)
            py_obj = resp.status_code
        elif action in ['create']:
            resp = lxca_rest().post_tasks(self.con.get_url(), self.con.get_session(), template)
            py_obj = resp
        else:
            resp = lxca_rest().get_tasks(self.con.get_url(), self.con.get_session(), job_uuid, includeChildren)
            py_obj = json.loads(resp.text)
            py_obj = {'TaskList': py_obj[:]}
        return py_obj

    def get_set_manifests( self, dict_handler = None ):
        sol_id = None
        filepath = None
                
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        if dict_handler:
            sol_id = next((item for item in [dict_handler.get('i') , dict_handler.get('id')] if item is not None),None)
            filepath = next((item for item in [dict_handler.get('f') , dict_handler.get('file')] if item is not None),None)
                        
        resp = lxca_rest().get_set_manifests(self.con.get_url(),self.con.get_session(),sol_id,filepath)
        
        try:
            py_obj = resp.json()
            return py_obj
        except AttributeError as ValueError:
            raise ValueError
        return py_obj



    def get_set_osimage(self, dict_handler = None):
        '''
        Reference URL: http://10.240.61.40:8131/help/topic/com.lenovo.lxca_restapis_all.doc/rest_apis_os_deployment_resources.html?cp=0_4_5
        commands construction:
        - osimages() 									<< GET  command   ::DONE
        - osimages(imageType=<DUD,BOOT,OS,OSPROFILE>)  	<< POST command   ::DONE
        - osimages(fileName=<>) 						<< GET  command   ::DONE
        - osimages(id=<>)								<< GET  command   ::DONE
        - osimages(id=<>, **kwargs)     				<< PUT/POST/DELETE command  :: DONE [TODO: its a complex post args]
        - osimages(jobid = <>)							<< POST command    ::DONE

        - osimages(remoteFileServers)					<< GET  command    ::DONE
        - osimages(remoteFileServers, **kwargs)			<< POST command    ::DONE
        - osimages(remoteFileServers, getId=<>)			<< GET  command    ::DONE
        - osimages(remoteFileServers, putId/deleteId=<>, **kwargs)	<< PUT/DELETE command  DONE

        - osimages(hostplatforms)						<< GET  command    ::DONE
        - osimages(hostplatforms, **kwargs)				<< PUT  command    ::DONE [TODO: jsonify complex args]

        - osimages(osdeployment)						<< PUT  command   :: DONE
        - osimages(osdeployment, **kwargs)				<< POST command   :: DONE [TODO: jsonify complex args]
        - osimages(connection)                          << GET  command   :: DONE
        - osimages(globalSettings)                      << GET  command   :: DONE
        - osimages(globalSettings, **kwargs)            << PUT  command   :: DONE

        '''
        get_method  = True
        id = None
        action = None
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")

        try:
            if dict_handler:
                osimages_info = next((item for item in [dict_handler.get('subcmd')] if item is not None), None)
                imageType = next((item for item in [dict_handler.get('t'), dict_handler.get('imagetype')] if item is not None), None)
                id = next((item for item in [dict_handler.get('i'), dict_handler.get('id')] if item is not None), None)
                action = next((item for item in [dict_handler.get('a'), dict_handler.get('action')] if item is not None), None)

                kwargs = next((item for item in [dict_handler.get('o'),
                                                dict_handler.get('osimages_dict')] if item is not None), None)

            if 'list' in osimages_info:
                resp = lxca_rest().list_osimage(self.con.get_url(), self.con.get_session())
            elif 'globalsettings' in osimages_info :
                resp = lxca_rest().osimage_globalsettings(self.con.get_url(), self.con.get_session(), kwargs)
            elif 'hostsettings' in osimages_info :
                if action:
                    if action in ['update']:
                        if not 'hosts' in kwargs:
                            raise Exception("Invalid argument: hosts detail is missing")
                        resp = lxca_rest().update_osimage_hostsettings(self.con.get_url(), self.con.get_session(), kwargs['hosts'])
                    elif action in ['create']:
                        if not 'hosts' in kwargs:
                            raise Exception("Invalid argument: hosts detail is missing")
                        resp = lxca_rest().create_osimage_hostsettings(self.con.get_url(), self.con.get_session(), kwargs['hosts'])
                    elif action in ['delete']:
                        resp = lxca_rest().delete_osimage_hostsettings(self.con.get_url(), self.con.get_session(), kwargs)
                else:
                    resp = lxca_rest().list_osimage_hostsettings(self.con.get_url(), self.con.get_session(), kwargs)
            elif 'hostplatforms' in osimages_info :
                resp = lxca_rest().osimage_hostplatforms(self.con.get_url(), self.con.get_session(), kwargs)
            elif 'import' in osimages_info:
                if not kwargs:
                    kwargs = {}
                kwargs['imageType'] = imageType
                resp = lxca_rest().osimage_import(self.con.get_url(), self.con.get_session(), kwargs)
            elif 'remotefileservers' in osimages_info:
                if not kwargs:
                    kwargs = {}
                resp = lxca_rest().osimage_remotefileservers(self.con.get_url(), self.con.get_session(), kwargs)

            elif 'delete' in osimages_info:
                if not kwargs:
                    kwargs = {}
                kwargs['id'] = id
                resp = lxca_rest().osimage_delete(self.con.get_url(), self.con.get_session(), kwargs)

            py_obj = resp.json()
            return py_obj

        except HTTPError as e:
            logger.error(e)
            if e.response.status_code == 403:
                return e.response.json()
            else:
                logger.error("Exception %s Occurred while calling REST API" %(e))
                raise e

        except ValueError as err:
            logger.error("Exception: Non json response: %s" %(str(err)))
            raise err
        except AttributeError as err:
            raise err
        return py_obj

    def get_set_resourcegroups(self, dict_handler = None):
        '''
    
        '''
        
        subcmd=uuid = name = desc = type = solutionVPD = members = criteria = None
        
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")

        # parsing dict_handler to fetch parameter    
        if dict_handler:
            uuid = next((item for item in [dict_handler.get('u') , dict_handler.get('uuid')] if item is not None),None)
            name = next((item for item in [dict_handler.get('n') , dict_handler.get('name')] if item is not None),None)
            desc = next((item for item in [dict_handler.get('d') , dict_handler.get('description')] if item is not None),None)
            type = next((item for item in [dict_handler.get('t') , dict_handler.get('type')] if item is not None),None)
            solutionVPD = next((item for item in [dict_handler.get('s') , dict_handler.get('solutionVPD')] if item is not None),None)
            members = next((item for item in [dict_handler.get('m') , dict_handler.get('members')] if item is not None),None)
            criteria = next((item for item in [dict_handler.get('c') , dict_handler.get('criteria')] if item is not None),None)
            subcmd = next(
                (item for item in [dict_handler.get('subcmd')] if item is not None), None)

        if 'list' in subcmd:
            resp = lxca_rest().list_resourcegroups(self.con.get_url(), self.con.get_session(), uuid)
        elif 'criteriaproperties' in subcmd:
            resp = lxca_rest().criteriaproperties_resourcegroups(self.con.get_url(), self.con.get_session())
            py_obj = json.loads(resp.text)
            py_obj = {'propertiesList': py_obj}
            return py_obj
        elif 'delete' in subcmd:
            resp = lxca_rest().delete_resourcegroups(self.con.get_url(), self.con.get_session(), uuid)
            if resp.status_code == 200:
                return "Deleted Successfully"
        elif 'create' in subcmd:
            if 'dynamic' in type:
                resp = lxca_rest().dynamic_resourcegroups(self.con.get_url(), self.con.get_session(), uuid, name, desc,
                                                  type, criteria)
            elif 'solution' in type:
                resp = lxca_rest().solution_resourcegroups(self.con.get_url(), self.con.get_session(), uuid, name,
                                                           desc, type, solutionVPD, members, criteria)
            else:
                raise Exception("Invalid argument: Type supported are dynamic and solution")
        elif 'update' in subcmd:
            if 'dynamic' in type:
                resp = lxca_rest().dynamic_resourcegroups(self.con.get_url(), self.con.get_session(), uuid, name,
                                                          desc,
                                                          type, criteria)
            elif 'solution' in type:
                resp = lxca_rest().solution_resourcegroups(self.con.get_url(), self.con.get_session(), uuid, name,
                                                           desc, type, solutionVPD, members, criteria)
            else:
                raise Exception("Invalid argument: Type supported are dynamic and solution")

        try:
            py_obj = json.loads(resp.text)
        except AttributeError as ValueError:
            raise ValueError
        return py_obj

    def get_set_rules(self, dict_handler=None):
        id = None
        rule = None

        if not self.con:
            raise ConnectionError("Connection is not Initialized.")

        if dict_handler:
            id = next((item for item in [dict_handler.get('i'), dict_handler.get('id')] if item is not None),
                        None)
            rule = next((item for item in [dict_handler.get('r'), dict_handler.get('rule')] if item is not None),
                             None)

        if rule:
            resp = lxca_rest().set_rules(self.con.get_url(), self.con.get_session(), rule)
        else:
            resp = lxca_rest().get_rules(self.con.get_url(), self.con.get_session(), id)
        py_obj = json.loads(resp.text)
        return py_obj

    def get_set_compositeResults(self, dict_handler=None):
        id = None
        query_solutionGroups = None
        solutionGroups = None
        targetResources = None
        all_rules = None

        if not self.con:
            raise ConnectionError("Connection is not Initialized.")

        if dict_handler:
            id = next((item for item in [dict_handler.get('i'), dict_handler.get('id')] if item is not None),
                        None)
            query_solutionGroups = next((item for item in
                [dict_handler.get('q'), dict_handler.get('query_solutionGroups')] if item is not None),
                      None)
            solutionGroups = next((item for item in [dict_handler.get('s'),
                dict_handler.get('solutionGroups')] if item is not None),
                             None)
            targetResources = next(
                (item for item in [dict_handler.get('t'), dict_handler.get('targetResources')] if item is not None),
                None)
            all_rules = next(
                (item for item in [dict_handler.get('a'), dict_handler.get('all_rules')] if item is not None),
                None)

        if all_rules or solutionGroups or targetResources:
            resp = lxca_rest().set_compositeResults(self.con.get_url(),
                    self.con.get_session(), solutionGroups, targetResources, all_rules)
        else:
            resp = lxca_rest().get_compositeResults(self.con.get_url(),
                    self.con.get_session(), id, query_solutionGroups)
        py_obj = json.loads(resp.text)
        return py_obj


    def get_set_storedcredentials(self, dict_handler=None):
        """
        Stored credential api handler
        :param dict_handler:
        :return:
        """
        id = None
        user_name = None
        description = None
        password = None
        delete_id = None

        if not self.con:
            raise ConnectionError("Connection is not Initialized.")

        if dict_handler:
            id = next((item for item in [dict_handler.get('i'), dict_handler.get('id')] if item is not None),
                        None)
            user_name = next((item for item in [dict_handler.get('u'), dict_handler.get('user_name')] if item is not None),
                             None)
            description = next((item for item in [dict_handler.get('d'), dict_handler.get('description')] if item is not None),
                             None)
            password = next(
                (item for item in [dict_handler.get('p'), dict_handler.get('password')] if item is not None),
                None)
            delete_id = next(
                (item for item in [dict_handler.get('delete_id')] if item is not None),
                None)

        if delete_id:
            resp = lxca_rest().delete_storedcredentials(self.con.get_url(), self.con.get_session(), delete_id)
            resp = json.loads(resp.text)
        elif id and (user_name or password or description):
            resp = lxca_rest().put_storedcredentials(self.con.get_url(), self.con.get_session(), id, user_name, password, description)
            resp = json.loads(resp.text)
        elif user_name and password:
            resp = lxca_rest().post_storedcredentials(self.con.get_url(), self.con.get_session(), user_name, password, description)
            py_obj = json.loads(resp.text)
            resp = {'storedcredentialsList': [py_obj['response']]}
        else:
            resp = lxca_rest().get_storedcredentials(self.con.get_url(), self.con.get_session(), id)
            py_obj = json.loads(resp.text)
            resp = {'storedcredentialsList': py_obj['response']}
        return resp

    def get_license(self, dict_handler=None):
        if not self.con:
            raise ConnectionError("Connection is not Initialized.")
        
        resp = lxca_rest().get_license(self.con.get_url(),self.con.get_session())
        py_obj = resp.json()
        return py_obj



'''
@since: 15 Jan 2016
@author: Girish Kumar <gkumar1@lenovo.com>
@license: Lenovo License
@copyright: Copyright 2016, Lenovo
@organization: Lenovo
@summary: Setup Script for PYLXCA
'''
import os, sys, re
from codecs import open

try:
    from setuptools import setup, find_packages
except ImportError:
    print ("setuptools is needed to run this file")
    print ("Try -- 'sudo pip install setuptools'")
    print ("Exiting ..")
    sys.exit(1)

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

with open('pylxca/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

if sys.version_info[:1] == (2,):
    setup(
        name                = "pylxca",
        version             = version,
        author              = "Girish Kumar, Prashant Bhosle",
        author_email        = "gkumar1@lenovo.com, pbhosle@lenovo.com",
        description         = ("It is tool/api to connect LXCA from command line"),
        license             = "Apache License 2.0",
        keywords            = "PYLXCA",
        url                 = "http://www.lenovo.com",
        packages            = ['pylxca','pylxca.pylxca_api','pylxca.pylxca_cmd'],
        long_description    = read('README'),
        long_description_content_type = 'text/x-rst',
        install_requires    = ['requests>=2.7.0', 'requests-toolbelt>=0.8.0',
            'mock>=2.0.0'],
        include_package_data= True,
        scripts             = ['lxca_shell'],
        #    data_files          = [('pylxca_api', ['pylxca/pylxca_api/lxca_logger.conf'])],
        classifiers         = [
            "Development Status :: 3 - Alpha",
            "Topic :: Utilities",
            "License :: OSI Approved :: Apache Software License",
        ],
    )
else:
    setup(
        name="pylxca",
        version=version,
        author="Girish Kumar, Prashant Bhosle",
        author_email="gkumar1@lenovo.com, pbhosle@lenovo.com",
        description=("It is tool/api to connect LXCA from command line"),
        license             = "Apache License 2.0",
        keywords="PYLXCA",
        url="http://www.lenovo.com",
        packages=['pylxca', 'pylxca.pylxca_api', 'pylxca.pylxca_cmd'],
        long_description=read('README'),
        long_description_content_type = 'text/x-rst',
        install_requires=['requests>=2.7.0', 'requests-toolbelt>=0.8.0', 'unittest2'],
        include_package_data=True,
        scripts=['lxca_shell'],
        #    data_files          = [('pylxca_api', ['pylxca/pylxca_api/lxca_logger.conf'])],
        classifiers=[
            "Development Status :: 3 - Alpha",
            "Topic :: Utilities",
            "License :: OSI Approved :: Apache Software License",
        ],
    )
